<?php
require_once('php/murder/renderer.inc'); 
require_once('php/murder/renderer_default.inc'); 

class Page
{
	static private $instance = false;

	private $user = false;
	private $renderer = false;

	#### page state ###################
	private $maxCache    = true;
	private $lastModified= false;
	private $vary	     = 'Accept';
	private $stylesheets = array();
	private $javascript  = array();
	private $keywords    = array();
	private $title       = false;
	private $statusCode  = 200;
	private $statusMessage= '';
	private $contentType = 'application/xhtml+xml';
	####################################

	private function __construct()
	{
		$this->user = safeGet($_GET,'_user');
		if($this->user && isset($_GET['_R']))
		{
			redirect($_SERVER['REQUEST_URI'] .'/');
			die;
		}
		$arguments = safeGet($_GET,'_arg');
		$sub       = safeGet($_GET,'_sub');
		if($arguments)
		{
			$arguments = explodea('/', $arguments);
		}
		$renderer = false;
		if($sub)
		{
			$renderer = PluginStore::getInstance()->getURLHandler($sub, $_SERVER['HTTP_ACCEPT']);
			if($renderer)
			{
				$this->renderer = new $renderer($this, $arguments);
			}
		}
		else
		{
			$renderer = PluginStore::getInstance()->getURLHandler('/', $_SERVER['HTTP_ACCEPT']);
			if($renderer)
			{
				// maybe there's a better content type
				$this->renderer = new $renderer($this, $arguments);
			}
			if(!$this->renderer)
			{
				$this->renderer = new DefaultRenderer($this, $arguments);
			}
		}
		if(!$this->renderer)
		{
			die("Shoot.");
		}
	}

	static public function getInstance()
	{
		if(!Page::$instance)
		{
			Page::$instance = new Page();
		}
		return Page::$instance;
	}

	#############################################
	## Rendering Methods
	#############################################
	public function render()
	{
		$r = $this->renderer;
		if(!$r) return;
		$r->processRequest($this);
		$r->sendHeaders($this);
		if($this->shouldRenderPage())
		{
			$r->startPage($this);
			$r->mainContent($this);
			$r->endPage($this);
		}
	}

	private function shouldRenderPage()
	{
		switch($this->statusCode)
		{
			case 304:
			case 406:
			case 429:
				return false;
		}
		return true;
	}

	public function setLastModified($time)
	{
		$this->lastModified = $time;
	}
	public function getLastModified($format=false) 
	{
		if($format)
		{
			if(!$this->lastModified) return false;
			return date('r',$this->lastModified);
		} else {
			return $this->lastModified;
		}
	}

	public function setTitle($title) { $this->title = $title; }

	public function getTitle() { 
		if($this->title) return $this->title; 
		return config('DEFAULT_PAGE_TITLE');
	}

	public function setContentType($type)
	{
		$this->contentType = $type;
	}
	public function getContentType() 
	{
		return $this->contentType;
	}

	public function setStatus($code, $message=false)
	{
		$this->statusCode = $code;
		$this->statusMessage= $message;
	}

	public function getStatusCode() { return $this->statusCode; }
	public function getStatusMessage()
	{
		if($this->statusMessage) return $this->statusMessage;
		switch($this->statusCode)
		{
			case 200: return 'Ok';
			case 300: return 'Multiple Choices';
			case 304: return 'Not Modified';
			case 400: return 'Bad Request';
			case 401: return 'Unauthorized';
			case 402: return 'Payment Required';
			case 403: return 'Forbidden';
			case 404: return 'Not Found';
			case 406: return 'None Acceptable';
			case 410: return 'Gone';
			case 415: return 'Unsupported Media Type';
			case 418: return 'I\'m a Teapot';
			case 429: return 'Too many requests.';
			case 500: return 'Internal Error';
			case 503: return 'Service Unavailable';
			default: return 'Unknknown.';
		}
	}

	public function setMaxAge($age) { $this->maxCache = $age; }
	public function getMaxAge() 
	{ 
		if($this->maxCache === true) return config('HTTP_CACHE_MAX_AGE');
		return $this->maxCache; 
	}
	public function setVary($vary) { $this->vary = $vary; }
	public function getVary() { return $this->vary; }

	#############################################
	## Common Methods
	#############################################
	public function getUser()
	{
		return UserStore::getInstance()->getUser($this->user);
	}

	public function myUrl()
	{
		return baseUrl() . $_SERVER['REQUEST_URI'];
	}
}
