<?php

class FoafRDFRenderer extends DefaultRenderer
{
	private $user;
	private $bIsGood;

	public function __construct($page, $arguments)
	{
		parent::__construct($page, $arguments);
		$this->user = $page->getUser();
		if(!$this->user || $this->user->isHidden()) 
		{
			$this->bIsGood = false;
		}
		else
		{
			$this->bIsGood = true; 
		}
	}

	public function mainContent($page)
	{
		if(!$this->bIsGood)
		{
			parent::mainContent($page);
			return;
		}
		$xml = FoafRDFRenderer::_buildContent($this->user);
		FoafRDFRenderer::_dumpXML($xml);
	}

	static private function _buildContent($user)
	{
		$r = FoafRDFRenderer::_buildNode('foaf:Person');
		FoafRDFRenderer::_setAttribute($r, 'rdf:ID', 'me');
		if($user->getName()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:name', $user->getName()));
		if($user->getLogin()) 
		{
			FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:nick', $user->getLogin()));
			if($domain = config('MAIL_DOMAIN'))
			{
				$mail = 'mailto:' .$user->getLogin().'@'.$domain;
				FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:mbox', $mail));
				FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:mbox_sha1sum', sha1($mail)));
			}
			if($domain = config('XMPP_DOMAIN'))
			{
				FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:jabberID', 'xmpp:'.$user->getLogin().'@'.$domain));
			}
		}
		if($user->getWorkPhone()) 
		{
			$n = FoafRDFRenderer::_buildNode('foaf:phone');
			FoafRDFRenderer::_setAttribute($n, 'rdf:resource', 'tel:'.$user->getWorkPhone());
			FoafRDFRenderer::_addChild($n, FoafRDFRenderer::_buildNode('dc:description', 'work'));
			FoafRDFRenderer::_addChild($r, $n);
		}
		if($user->getHomePhone()) 
		{
			$n = FoafRDFRenderer::_buildNode('foaf:phone');
			FoafRDFRenderer::_setAttribute($n, 'rdf:resource', 'tel:'.$user->getHomePhone());
			FoafRDFRenderer::_addChild($n, FoafRDFRenderer::_buildNode('dc:description', 'home'));
			FoafRDFRenderer::_addChild($r, $n);
		}
		if($user->getPlan()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:plan', $user->getPlan()));
		
		return $r;
	}

	static private function _buildNode($name, $value = '')
	{
		return array( 'name' => $name, 'value' => $value);
	}

	static private function _setAttribute(&$node, $name, $value)
	{
		if(!isset($node['attrs'])) $node['attrs'] = array();
		$node['attrs'][$name] = $value;
	}

	static private function _addChild(&$parent, &$child)
	{
		if(!isset($parent['children']))
			$parent['children'] = array();
		$parent['children'][] = $child;
	}

	static private function _dumpXML($xml)
	{
		if(!$xml || !is_array($xml)) return;
		$n = htmlspecialchars(safeGet($xml, 'name'));
		$a = safeGet($xml, 'attrs');
		$v = htmlspecialchars(safeGet($xml, 'value'));
		$c = safeGet($xml, 'children');
		print "<${n}";
		if($a && is_array($a)) foreach($a as $ak => $av) {
			print ' ' . htmlspecialchars($ak) . '=' . '"' . htmlspecialchars($av) .'"';
		}
		print ">${v}";
		if($c && is_array($c)) foreach($c as $p) FoafRDFRenderer::_dumpXML($p);
		print "</${n}>";
	}

	public function sendHeaders($page)
	{
		if($this->bIsGood) 
		{
			$page->setContentType('application/rdf+xml');
		}
		parent::sendHeaders($page);
	}

	public function startPage($page)
	{
		if(!$this->bIsGood) 
		{
			parent::startPage($page);
			return;
		}
?>
<rdf:RDF
      xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
      xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
      xmlns:foaf="http://xmlns.com/foaf/0.1/"
      xmlns:admin="http://webns.net/mvcb/">
<foaf:PersonalProfileDocument rdf:about="<?php echo htmlspecialchars($page->myUrl());?>">
  <foaf:maker rdf:resource="#me"/>
  <foaf:primaryTopic rdf:resource="#me"/>
  <admin:generatorAgent rdf:resource="http://matt.wronka.org/stuff/projects/murder/"/>
  <admin:errorReportsTo rdf:resource="mailto:murder@projects.matt.wronka.org"/>
</foaf:PersonalProfileDocument>
<?php
	}

	public function endPage($page)
	{
		if(!$this->bIsGood) 
		{
			parent::endPage($page);
			return;
		}
		?></rdf:RDF><?php
	}
}

registerp(array('application/rdf+xml' => "FoafRDFRenderer"), "/");

?>
